package com.gree.mall.manager.logic.workorder;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.es.*;
import com.gree.mall.manager.bean.listvo.param.WorkOrderZfireParam;
import com.gree.mall.manager.bean.listvo.workorder.OrderBaseVO;
import com.gree.mall.manager.enums.MaterialExamineStatusEnum;
import com.gree.mall.manager.es.OrderBaseEsRepository;
import com.gree.mall.manager.es.OrderWorkerEsRepository;
import com.gree.mall.manager.es.SettlementOrderEsRepository;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.PgOrderBaseService;
import com.gree.mall.manager.plus.service.PgOrderWorkerService;
import com.gree.mall.manager.plus.service.SettlementOrderService;
import com.gree.mall.manager.plus.service.WorkerOrderService;
import com.gree.mall.manager.utils.CommonUtils;
import com.gree.mall.manager.utils.StringUtil;
import com.gree.mall.manager.zfire.bean.QueryParamBean;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.Scroll;
import org.elasticsearch.search.aggregations.AbstractAggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.Aggregations;
import org.elasticsearch.search.aggregations.bucket.filter.FilterAggregationBuilder;
import org.elasticsearch.search.aggregations.bucket.filter.ParsedFilter;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.*;
import org.springframework.data.elasticsearch.core.document.Document;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class OrderBaseEsLogic {


    private final OrderFlagLogic orderFlagLogic;

    private final WorkerOrderService workerOrderService;

    private final ElasticsearchRestTemplate elasticsearchRestTemplate;

    private final CommonLogic commonLogic;
    private final OrderBaseEsRepository orderBaseEsRepository;
    private final PgOrderBaseService pgOrderBaseService;
    private final SettlementOrderService settlementOrderService;
    private final SettlementOrderEsRepository settlementOrderEsRepository;
    private final OrderWorkerEsRepository orderWorkerEsRepository;
    private final PgOrderWorkerService pgOrderWorkerService;

    public IPage<OrderBaseVO> orderBaseEsList(WorkOrderZfireParam zfireParamBean, HttpServletRequest request) {

        AdminUserCom adminUser = commonLogic.getAdminUser();

        String year = CommonUtils.getYear(request);

        BoolQueryBuilder queryBuilder1 = QueryBuilders.boolQuery();
        this.montage(queryBuilder1,zfireParamBean);

        if (CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds())) {
            queryBuilder1.must(
                    QueryBuilders.boolQuery().should(QueryBuilders.termsQuery("create_websit_id", adminUser.getAdminWebsitIds()))
                    .should(QueryBuilders.termsQuery("websit_id", adminUser.getAdminWebsitIds()))
                    );

        }

        List<String> orderBaseId = new ArrayList<>();

        if (zfireParamBean.getIsWait() != null && zfireParamBean.getIsWait()){
            List<String> collect = workerOrderService.lambdaQuery()
                    .eq(WorkerOrder::getPayStatus, "WAIT")
                    .in(CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds()),WorkerOrder::getWebsitId,  adminUser.getAdminWebsitIds())
                    .ne(WorkerOrder::getWorkerOrderId, "").select(WorkerOrder::getWorkerOrderId)
                    .list().stream().map(WorkerOrder::getWorkerOrderId).collect(Collectors.toList());

            if (CollectionUtils.isEmpty(collect))
                return new Page<>();

            orderBaseId.addAll(collect);


        }

        if (!CollectionUtils.isEmpty(orderBaseId)){
            queryBuilder1.must(QueryBuilders.termsQuery("id",orderBaseId));
        }



        NativeSearchQuery build = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder1)
                .withSort(SortBuilders.fieldSort("create_time").order(SortOrder.DESC))
                .withPageable(PageRequest.of(zfireParamBean.getPageNum() -1, zfireParamBean.getPageSize()))
                .build();

        build.setTrackTotalHits(true);

        List<OrderBaseEs> orderBaseEs = new ArrayList<>();
        IPage<OrderBaseVO> page = new Page<>();



        if (year.equals("2023")) {
            SearchHits<OrderBase2023Es> orderBaseEsSearchHits = elasticsearchRestTemplate.search(build, OrderBase2023Es.class);
            SearchPage<OrderBase2023Es> searchHits2023 = SearchHitSupport.searchPageFor(orderBaseEsSearchHits, build.getPageable());

            for (SearchHit<OrderBase2023Es> searchHit : searchHits2023.getContent()) {
                orderBaseEs.add(BeanUtil.toBean(searchHit.getContent(),OrderBaseEs.class));
            }


            page.setTotal(searchHits2023.getSearchHits().getTotalHits());
            page.setCurrent(zfireParamBean.getPageNum());
            page.setSize(zfireParamBean.getPageSize());
            page.setRecords(BeanUtil.copyToList(orderBaseEs,OrderBaseVO.class));
        }else

        if (year.equals("2024")) {
            SearchHits<OrderBase2024Es> orderBaseEsSearchHits = elasticsearchRestTemplate.search(build, OrderBase2024Es.class);
            SearchPage<OrderBase2024Es> searchHits2024 = SearchHitSupport.searchPageFor(orderBaseEsSearchHits, build.getPageable());

            for (SearchHit<OrderBase2024Es> searchHit : searchHits2024.getContent()) {
                orderBaseEs.add(BeanUtil.toBean(searchHit.getContent(),OrderBaseEs.class));
            }


            page.setTotal(searchHits2024.getSearchHits().getTotalHits());
            page.setCurrent(zfireParamBean.getPageNum());
            page.setSize(zfireParamBean.getPageSize());
            page.setRecords(BeanUtil.copyToList(orderBaseEs,OrderBaseVO.class));
        }else


        if (year.equals("2022") || Convert.toInt(year) < 2022) {
            SearchHits<OrderBase2022Es> orderBaseEsSearchHits = elasticsearchRestTemplate.search(build, OrderBase2022Es.class);
            SearchPage<OrderBase2022Es> searchHits2022 = SearchHitSupport.searchPageFor(orderBaseEsSearchHits, build.getPageable());

            for (SearchHit<OrderBase2022Es> searchHit : searchHits2022.getContent()) {
                orderBaseEs.add(BeanUtil.toBean(searchHit.getContent(),OrderBaseEs.class));
            }


            page.setTotal(searchHits2022.getSearchHits().getTotalHits());
            page.setCurrent(zfireParamBean.getPageNum());
            page.setSize(zfireParamBean.getPageSize());
            page.setRecords(BeanUtil.copyToList(orderBaseEs,OrderBaseVO.class));
        }else {
            SearchHits<OrderBaseEs> orderBaseEsSearchHits = elasticsearchRestTemplate.search(build, OrderBaseEs.class);
            SearchPage<OrderBaseEs> searchHits = SearchHitSupport.searchPageFor(orderBaseEsSearchHits, build.getPageable());

            for (SearchHit<OrderBaseEs> searchHit : searchHits.getContent()) {
                orderBaseEs.add(searchHit.getContent());
            }
            page.setTotal(searchHits.getSearchHits().getTotalHits());
            page.setCurrent(zfireParamBean.getPageNum());
            page.setSize(zfireParamBean.getPageSize());
            page.setRecords(BeanUtil.copyToList(orderBaseEs,OrderBaseVO.class));
        }




        //服务单标识
        if (zfireParamBean.getPageSize() != -1 && page.getTotal() > 0 && page.getRecords().size() > 0) {
            List<String> orderBaseIds = page.getRecords()
                    .stream()
                    .map(OrderBaseVO::getId)
                    .collect(Collectors.toList());
            Map<String, List<PgOrderFlag>> mapList = orderFlagLogic.list(orderBaseIds,year);

            List<WorkerOrder> workerOrderList = workerOrderService.lambdaQuery().in(WorkerOrder::getWorkerOrderId, orderBaseIds)
                    .eq(WorkerOrder::getPayStatus, MaterialExamineStatusEnum.PAID.getKey())
                    .select(WorkerOrder::getWorkerOrderId, WorkerOrder::getTotalAmount).list();

            for (OrderBaseVO orderBaseVO : page.getRecords()) {
                orderBaseVO.setOrderFlags(mapList.get(orderBaseVO.getId()));

                List<WorkerOrder> workerOrders = workerOrderList.stream()
                        .filter(item -> item.getWorkerOrderId().equals(orderBaseVO.getId())).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(workerOrderList)) {
                    BigDecimal bigDecimal = workerOrders.stream().map(WorkerOrder::getTotalAmount).reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                    orderBaseVO.setWorkerPrice(bigDecimal);
                }

            }
        }
        return page;
    }



    private void montage(BoolQueryBuilder queryBuilder, WorkOrderZfireParam workOrderZfireParam) {

        List<QueryParamBean> params = workOrderZfireParam.getParams();
        if (params == null || params.size() == 0) {
            return;
        }

        List<QueryBuilder> should = queryBuilder.should();
        for (QueryParamBean paramBean : params) {
            if (paramBean.getValue() == null)
                continue;

            String param = "";
            if (paramBean.getParam().split("\\.").length > 1){
                param = paramBean.getParam().split("\\.")[1];
            }else {
                param = paramBean.getParam();
            }
            if (paramBean.getValue() instanceof ArrayList) {
                List<String> values = (List<String>) paramBean.getValue();
                if (values != null && values.size() > 0) {
                    List<String> join = new ArrayList<>();
                    for (String s : values) {
                        join.add(s);
                    }

                    queryBuilder.must(QueryBuilders.termsQuery(param,join));
                    continue;
                }
            } else {
                String value = "";
                if (paramBean.getValue() instanceof Boolean) {
                    value = paramBean.getValue().toString();
                } else {
                    value = paramBean.getValue().toString();
                }
                if (StringUtils.isBlank(value))
                    continue;

                if (param.equals("order_status") && value.equals("DYY")
                ){

                    queryBuilder.mustNot(QueryBuilders.existsQuery("appointment_time"));
                    queryBuilder.mustNot(QueryBuilders.termsQuery("order_status", "YWG", "GCSZX", "WDWG", "YWGO", "YQX", "FWZT", "YCGB","YZP",
                                    "FWQX", "FL", "YJS", "LRCD", "DSHPG", "CJ", "YPD", "DXSPD", "DZBPG", "DWDSPGP", "DXSSPGP", "DTJXSSPGP", "DZBSPGP"));

                    continue;
                }

                if (param.equals("order_status")
                        &&  value.equals("YCD")
                ){
                    queryBuilder.must(QueryBuilders.termQuery("is_exception",true));
                    queryBuilder.mustNot(QueryBuilders.termsQuery("order_status", "YWG", "YWGO", "YJS"));
                    continue;
                }

                if (param.equals("order_status")
                        && value.equals("DJD")
                ){

                    queryBuilder.must(QueryBuilders.boolQuery()
                            .mustNot(QueryBuilders.termsQuery("order_status","YQX", "FWQX", "FL", "FWZT", "YCGB","YZP"))
                            .must(QueryBuilders.boolQuery().should(QueryBuilders.termQuery("is_meet", false))
                                    .should(QueryBuilders.termsQuery("order_status", "DJD"))));
                    continue;
                }


                if (param.equals("order_status")
                        && value.equals("PJSQZ")
                ){
                    queryBuilder.must(QueryBuilders.termQuery("parts_apply_status","ING"));
                    continue;
                }

                if (param.equals("order_status")
                        && value.equals("PJYDH")
                ){
                    queryBuilder.must(QueryBuilders.termQuery("parts_apply_status","END"));
                    continue;
                }


                if (param.equals("order_status")
                        && value.equals("PJYQX")
                ){
                    queryBuilder.must(QueryBuilders.termQuery("parts_apply_status","CANCEL"));
                    continue;
                }

                if (StringUtils.equals(paramBean.getCompare(), "like")) {
                    queryBuilder.must(QueryBuilders.wildcardQuery(param,"*"+value+"*"));
                    continue;
                }

                if (value.equals("true") || value.equals("false")) {

                    queryBuilder.must(QueryBuilders.termQuery(param,value.equals("true")?true:false));
                    continue;
                } else {
                    if (paramBean.getCompare().equals(">=")){

                        queryBuilder.must(QueryBuilders.rangeQuery(param).gte(value));
                        continue;
                    }
                    if (paramBean.getCompare().equals("<=")){
                        queryBuilder.must(QueryBuilders.rangeQuery(param).lte(value));
                        continue;
                    }
                    queryBuilder.must(QueryBuilders.termQuery(param,value));
                    continue;
                }
            }
        }


    }

        public static String replaceValue(String value) {
            return value.replaceAll("'", "").replaceAll("\"", "")
                    .replaceAll(" or ", "").replaceAll(" union ", "");
        }


    public void test() {

    /*    boolean a = elasticsearchRestTemplate.indexOps(OrderBaseEs2022Date.class).delete();
        IndexOperations indexOperations1 = elasticsearchRestTemplate.indexOps(OrderBaseEs2022Date.class);
        if (!indexOperations1.exists()) {
            indexOperations1.create();
            Document document = indexOperations1.createMapping();
            indexOperations1.putMapping(document);
        }*/

/*        boolean a = elasticsearchRestTemplate.indexOps(OrderBaseEs2023Date.class).delete();
        IndexOperations indexOperations1 = elasticsearchRestTemplate.indexOps(OrderBaseEs2023Date.class);
        if (!indexOperations1.exists()) {
            indexOperations1.create();
            Document document = indexOperations1.createMapping();
            indexOperations1.putMapping(document);
        }*/
        boolean a = elasticsearchRestTemplate.indexOps(OrderBaseEs2024Date.class).delete();
        IndexOperations indexOperations1 = elasticsearchRestTemplate.indexOps(OrderBaseEs2024Date.class);
        if (!indexOperations1.exists()) {
            indexOperations1.create();
            Document document = indexOperations1.createMapping();
            indexOperations1.putMapping(document);
        }


/*        boolean a = elasticsearchRestTemplate.indexOps(OrderBaseEsDate.class).delete();
        IndexOperations indexOperations1 = elasticsearchRestTemplate.indexOps(OrderBaseEsDate.class);
        if (!indexOperations1.exists()) {
            indexOperations1.create();
            Document document = indexOperations1.createMapping();
            indexOperations1.putMapping(document);
        }*/


/*
        List<PgOrderBase> list = pgOrderBaseService.lambdaQuery().list();

       // List<OrderBaseEsDate> orderBaseEsDates = BeanUtil.copyToList(list, OrderBaseEsDate.class);
        for (PgOrderBase pgOrderBase : list) {
            OrderBaseEs orderBaseEs = BeanUtil.toBean(pgOrderBase, OrderBaseEs.class);

            orderBaseEs.setDispatchTime(pgOrderBase.getDispatchTime() != null? DateUtil.format(pgOrderBase.getDispatchTime(),"yyyy-MM-dd HH:mm:ss"):null);
            orderBaseEs.setWorkerReceTime(pgOrderBase.getWorkerReceTime() != null? DateUtil.format(pgOrderBase.getWorkerReceTime(),"yyyy-MM-dd HH:mm:ss"):null);
            orderBaseEs.setDispatchWebsitTime(pgOrderBase.getDispatchWebsitTime() != null? DateUtil.format(pgOrderBase.getDispatchWebsitTime(),"yyyy-MM-dd HH:mm:ss"):null);

            orderBaseEs.setLastOperatorTime(pgOrderBase.getLastOperatorTime() != null? DateUtil.format(pgOrderBase.getLastOperatorTime(),"yyyy-MM-dd HH:mm:ss"):null);
            orderBaseEs.setAppointmentTime(pgOrderBase.getAppo() != null? DateUtil.format(pgOrderBase.getLastOperatorTime(),"yyyy-MM-dd HH:mm:ss"):null);

            orderBaseEsList.add(orderBaseEs);
        }

      // orderBaseEsRepository.saveAll(orderBaseEsDates);
*/


    /*    boolean a = elasticsearchRestTemplate.indexOps(PgOrderWorkerEs.class).delete();
        IndexOperations indexOperations1 = elasticsearchRestTemplate.indexOps(PgOrderWorkerEs.class);
        if (!indexOperations1.exists()) {
            indexOperations1.create();
            Document document = indexOperations1.createMapping();
            indexOperations1.putMapping(document);
        }*/
    }

    public List<Map<String, Object>> countStatus(String startTime, String endTime, String orderSmallType, String orderSmallTypeText,HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser();

        String year = CommonUtils.getYear(request);
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();

        List<Map<String, Object>> maps = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds())) {
            queryBuilder.must(
                    QueryBuilders.boolQuery().should(QueryBuilders.termsQuery("create_websit_id", adminUser.getAdminWebsitIds()))
                            .should(QueryBuilders.termsQuery("websit_id", adminUser.getAdminWebsitIds()))
            );
        }

        if (!StringUtil.isEmpty(orderSmallTypeText))
            queryBuilder.must(QueryBuilders.termsQuery("order_small_type_text",orderSmallTypeText));


        if (startTime != null)
            queryBuilder.must(QueryBuilders.rangeQuery("create_time")
                .gte(startTime)
                .lte(endTime)
        );


        //待预约
        NativeSearchQuery searchQuery = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("DYY", QueryBuilders.boolQuery()
                        .mustNot(QueryBuilders.existsQuery("appointment_time"))
                        .mustNot(QueryBuilders.termsQuery("order_status", "YWG", "GCSZX", "WDWG", "YWGO", "YQX", "FWZT", "YCGB","YZP",
                                "FWQX", "FL", "YJS", "LRCD", "DSHPG", "CJ", "YPD", "DXSPD", "DZBPG", "DWDSPGP", "DXSSPGP", "DTJXSSPGP", "DZBSPGP"))
                        .must(QueryBuilders.rangeQuery("create_time")
                                .gte(startTime)
                                .lte(endTime)
                        ))))
                .build();

        SearchHits response = this.search(searchQuery,year);
        HashMap<String, Object> dyy = new HashMap<>();
        dyy.put("total", this.getWorkerOrderCountResult(response.getAggregations(), "DYY"));
        dyy.put("orderStatus", "DYY");
        maps.add(dyy);



        //待抢单
        NativeSearchQuery searchQueryDqd = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("DQD", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termQuery("order_status","DQD")))))
                .build();

        SearchHits responsedqd = this.search(searchQueryDqd,year);

        HashMap<String, Object> dqd = new HashMap<>();
        dqd.put("total",this.getWorkerOrderCountResult(responsedqd.getAggregations(),"DQD"));
        dqd.put("orderStatus","DQD");
        maps.add(dqd);


        //待商户派工
        NativeSearchQuery searchQueryDshpg = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("DSHPG", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termsQuery("order_status","DSHPG","CJ","YPD","DXSPD","DZBPG","DWDSPGP","DXSSPGP",
                                "DTJXSSPGP","DZBSPGP","DFZXPD","DFZXSPGP")))))
                .build();
        SearchHits responseDshpg =this.search(searchQueryDshpg,year);

        HashMap<String, Object> dshpg = new HashMap<>();
        dshpg.put("total",this.getWorkerOrderCountResult(responseDshpg.getAggregations(),"DSHPG"));
        dshpg.put("orderStatus","DSHPG");

        maps.add(dshpg);


        //待网点派工
        NativeSearchQuery searchQueryDwdpg = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("DWDPG", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termsQuery("order_status","DWDPG","DWDPD")))))
                .build();
        SearchHits responseDwdpg = this.search(searchQueryDwdpg,year);

        HashMap<String, Object> DWDPG = new HashMap<>();
        DWDPG.put("total",this.getWorkerOrderCountResult(responseDwdpg.getAggregations(),"DWDPG"));
        DWDPG.put("orderStatus","DWDPG");
        maps.add(DWDPG);


        //待接单
        NativeSearchQuery searchQueryDJD = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("DJD", QueryBuilders.boolQuery()
                        .mustNot(QueryBuilders.termsQuery("order_status","YQX", "FWQX", "FL", "FWZT", "YCGB","YZP"))
                        .must(QueryBuilders.boolQuery().should(QueryBuilders.termQuery("is_meet", false))
                        .should(QueryBuilders.termsQuery("order_status", "DJD")))
                        )))
                .build();
        SearchHits responseDJD = this.search(searchQueryDJD,year);

        HashMap<String, Object> DJD = new HashMap<>();
        DJD.put("total",this.getWorkerOrderCountResult(responseDJD.getAggregations(),"DJD"));
        DJD.put("orderStatus","DJD");

        maps.add(DJD);

        //服务中
        NativeSearchQuery searchQueryfwz = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("FWZ", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termsQuery("order_status","FWZ","GCSZT","BFWG",
                                "DSM","WDBH", "XSBH","TJXSBH","ZBBH","FZXBH")))))
                .build();
        SearchHits responseFWZ = this.search(searchQueryfwz,year);

        HashMap<String, Object> FWZ = new HashMap<>();
        FWZ.put("total",this.getWorkerOrderCountResult(responseFWZ.getAggregations(),"FWZ"));
        FWZ.put("orderStatus","FWZ");

        maps.add(FWZ);


        //异常单
        NativeSearchQuery searchQueryYCD = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("YCD", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termQuery("is_exception",true))
                        .mustNot(QueryBuilders.termsQuery("order_status", "YWG", "YWGO", "YJS")))))
                .build();
        SearchHits responseYCD = this.search(searchQueryYCD,year);

        HashMap<String, Object> YCD = new HashMap<>();
        YCD.put("total",this.getWorkerOrderCountResult(responseYCD.getAggregations(),"YCD"));
        YCD.put("orderStatus","YCD");

        maps.add(YCD);

        //已完工待结算
        NativeSearchQuery searchQueryYWGDJS = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("YWG", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termsQuery("order_status","YWG","GCSZX","WDWG","YWGO")))))
                .build();
        SearchHits responseYWG = this.search(searchQueryYWGDJS,year);

        HashMap<String, Object> YWG = new HashMap<>();
        YWG.put("total",this.getWorkerOrderCountResult(responseYWG.getAggregations(),"YWG"));
        YWG.put("orderStatus","YWG");

        maps.add(YWG);


        //已结算
        NativeSearchQuery searchQueryYJS= new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("YJS", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termsQuery("order_status","YJS","LRCD")))))
                .build();
        SearchHits responseYJS = this.search(searchQueryYJS,year);

        HashMap<String, Object> YJS = new HashMap<>();
        YJS.put("total",this.getWorkerOrderCountResult(responseYJS.getAggregations(),"YJS"));
        YJS.put("orderStatus","YJS");

        maps.add(YJS);

        //已取消
        NativeSearchQuery searchQueryYQX = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("YQX", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termsQuery("order_status","YZP","YQX","FWZT","YCGB","FWQX","FL")))))
                .build();
        SearchHits responseYQX = this.search(searchQueryYQX,year);

        HashMap<String, Object> YQX = new HashMap<>();
        YQX.put("total",this.getWorkerOrderCountResult(responseYQX.getAggregations(),"YQX"));
        YQX.put("orderStatus","YQX");

        maps.add(YQX);


        //配件申请中
        NativeSearchQuery searchQueryPJSQZ = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("PJSQZ", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termQuery("parts_apply_status","ING")))))
                .build();
        SearchHits responsePJSQZ = this.search(searchQueryPJSQZ,year);

        HashMap<String, Object> PJSQZ = new HashMap<>();
        PJSQZ.put("total",this.getWorkerOrderCountResult(responsePJSQZ.getAggregations(),"PJSQZ"));
        PJSQZ.put("orderStatus","PJSQZ");

        maps.add(PJSQZ);


        //配件已到货
        NativeSearchQuery searchQueryPJYDH = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("PJYDH", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termQuery("parts_apply_status","END")))))
                .build();
        SearchHits responsePJYDH =  this.search(searchQueryPJYDH,year);

        HashMap<String, Object> PJYDH = new HashMap<>();
        PJYDH.put("total",this.getWorkerOrderCountResult(responsePJYDH.getAggregations(),"PJYDH"));
        PJYDH.put("orderStatus","PJYDH");

        maps.add(PJYDH);


        //配件已取消
        NativeSearchQuery searchQueryPJYQX = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder)
                .withPageable(PageRequest.of(1, 1))
                .addAggregation(this.getAggs(AggregationBuilders.filter("PJYQX", QueryBuilders.boolQuery()
                        .must(QueryBuilders.termQuery("parts_apply_status","CANCEL")))))
                .build();
        SearchHits responsePJYQX = this.search(searchQueryPJYQX,year);

        HashMap<String, Object> PJYQX = new HashMap<>();
        PJYQX.put("total",this.getWorkerOrderCountResult(responsePJYQX.getAggregations(),"PJYQX"));
        PJYQX.put("orderStatus","PJYQX");

        maps.add(PJYQX);

        List<String> orderBaseId = new ArrayList<>();

        HashMap<String, Object> DZF = new HashMap<>();
        List<String> collect = workerOrderService.lambdaQuery()
                .eq(WorkerOrder::getPayStatus, "WAIT")
                .between(!StringUtil.isEmpty(startTime),WorkerOrder::getCreateTime, startTime,endTime)
                .ne(WorkerOrder::getWorkerOrderId, "").select(WorkerOrder::getWorkerOrderId)
                .list().stream().map(WorkerOrder::getWorkerOrderId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(collect)) {

            orderBaseId.addAll(collect);
            //待支付
            NativeSearchQuery searchQueryDZF = new NativeSearchQueryBuilder()
                    .withQuery(queryBuilder)
                    .withPageable(PageRequest.of(1, 1))
                    .addAggregation(this.getAggs(AggregationBuilders.filter("DZF", QueryBuilders.boolQuery()
                            .must(QueryBuilders.termsQuery("id",orderBaseId))
                    )))
                    .build();
            SearchHits responseDZF = this.search(searchQueryDZF,year);
            DZF.put("total",this.getWorkerOrderCountResult(responseDZF.getAggregations(),"DZF"));
            DZF.put("orderStatus","DZF");

            maps.add(DZF);

        }

        return maps;
    }


    private SearchHits search(NativeSearchQuery searchQuery,String year) {

        if (year.equals("2023")) {
            SearchHits response = elasticsearchRestTemplate.search(searchQuery, OrderBase2023Es.class);
             return response;
        }
        if (year.equals("2024")) {
            SearchHits response = elasticsearchRestTemplate.search(searchQuery, OrderBase2024Es.class);
            return response;
        }

        if (year.equals("2022") || Convert.toInt(year) < 2022) {
            SearchHits response = elasticsearchRestTemplate.search(searchQuery, OrderBase2022Es.class);
            return response;
        }else {
            SearchHits response = elasticsearchRestTemplate.search(searchQuery, OrderBaseEs.class);
            return response;
        }
    }

    public AbstractAggregationBuilder<?> getAggs(FilterAggregationBuilder filter) {
        return filter.subAggregation(AggregationBuilders.sum("total_mac_sum").field("total_num"))
                .subAggregation(AggregationBuilders.terms("order_type_group").field("order_type")
                        .subAggregation(AggregationBuilders.sum("mac_sum").field("total_num")));
    }
    private long getWorkerOrderCountResult(Aggregations aggregations, String name) {
        StringBuffer sb = new StringBuffer();
        ParsedFilter parsedFilter = aggregations.get(name);
        //总服务单数
        long totalOrderNums = parsedFilter.getDocCount();

        return totalOrderNums;
    }

    public void esSync() {

        List<PgOrderBase> list = pgOrderBaseService.lambdaQuery()
                .gt(PgOrderBase::getUpdateTime,DateUtil.offsetMinute(new Date(),-5))
                .list();

        List<OrderBaseEsDate> orderBaseEsDates = BeanUtil.copyToList(list, OrderBaseEsDate.class);


        orderBaseEsRepository.saveAll(orderBaseEsDates);
    }

    public void esSync1() {
        List<SettlementOrder> list = settlementOrderService.lambdaQuery()
                .gt(SettlementOrder::getUpdateTime, DateUtil.offsetMinute(new Date(), -5))
                .list();

        List<SettlementOrderEs> orderBaseEsDates = BeanUtil.copyToList(list, SettlementOrderEs.class);


        settlementOrderEsRepository.saveAll(orderBaseEsDates);
    }

    public void del() {
    }

    public void esSync2() {

        List<PgOrderWorker> list = pgOrderWorkerService.lambdaQuery()
                .gt(PgOrderWorker::getUpdateTime,DateUtil.offsetMinute(new Date(),-5))
                .list();

        List<PgOrderWorkerEs> pgOrderWorkerEs = BeanUtil.copyToList(list, PgOrderWorkerEs.class);


        orderWorkerEsRepository.saveAll(pgOrderWorkerEs);
    }

    public List<OrderBaseVO> listExport(WorkOrderZfireParam zfireParamBean,HttpServletRequest request) {

        AdminUserCom adminUser = commonLogic.getAdminUser();

        String year = CommonUtils.getYear(request);

        BoolQueryBuilder queryBuilder1 = QueryBuilders.boolQuery();
        this.montage(queryBuilder1,zfireParamBean);

        if (CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds())) {
            queryBuilder1.must(
                    QueryBuilders.boolQuery().should(QueryBuilders.termsQuery("create_websit_id", adminUser.getAdminWebsitIds()))
                            .should(QueryBuilders.termsQuery("websit_id", adminUser.getAdminWebsitIds()))
            );

        }

        List<String> orderBaseId = new ArrayList<>();

        if (zfireParamBean.getIsWait() != null && zfireParamBean.getIsWait()){
            List<String> collect = workerOrderService.lambdaQuery()
                    .eq(WorkerOrder::getPayStatus, "WAIT")
                    .in(CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds()),WorkerOrder::getWebsitId,  adminUser.getAdminWebsitIds())
                    .ne(WorkerOrder::getWorkerOrderId, "").select(WorkerOrder::getWorkerOrderId)
                    .list().stream().map(WorkerOrder::getWorkerOrderId).collect(Collectors.toList());

            if (CollectionUtils.isEmpty(collect))
                return new ArrayList<>();

            orderBaseId.addAll(collect);


        }

        if (!CollectionUtils.isEmpty(orderBaseId)){
            queryBuilder1.must(QueryBuilders.termsQuery("id",orderBaseId));
        }



        NativeSearchQuery query  = new NativeSearchQueryBuilder()
                .withQuery(queryBuilder1)
                .withSort(SortBuilders.fieldSort("create_time").order(SortOrder.DESC))
                .withPageable(PageRequest.of(0, 3000))
                .build();



        // 2. 初始化 Scroll

        List<OrderBaseVO> orderBaseVOS1 = new ArrayList<>();

        if (year.equals("2023")) {

            List<OrderBase2023Es> orderBaseVOS = new ArrayList<>();
            SearchScrollHits<OrderBase2023Es> scrollHits = elasticsearchRestTemplate.searchScrollStart(
                    60000, // Scroll 上下文保持时间（毫秒）
                    query,
                    OrderBase2023Es.class,
                    IndexCoordinates.of("pg_order_base_2023")
            );

            String scrollId = scrollHits.getScrollId();
            int batch = 0;

            if (scrollHits.getTotalHits() > 60000)
                throw  new RemoteServiceException("数据超过六万上限，请分批导出");


            try {
                while (!scrollHits.isEmpty()) {
                    // 3. 处理当前批次数据
                    scrollHits.forEach(hit -> {
                        OrderBase2023Es entity = hit.getContent();
                        orderBaseVOS.add(entity);
                    });

                    batch++;

                    // 4. 获取下一批次
                    scrollHits = elasticsearchRestTemplate.searchScrollContinue(
                            scrollId,
                            60000,
                            OrderBase2023Es.class,
                            IndexCoordinates.of("pg_order_base_2023")
                    );
                    scrollId = scrollHits.getScrollId(); // 更新 scrollId
                }
            } finally {
                // 5. 清理 Scroll 上下文
                if (scrollId != null) {
                    List<String> scroll = new ArrayList<>();
                    scroll.add(scrollId);
                    elasticsearchRestTemplate.searchScrollClear(scroll);
                }
            }

            orderBaseVOS1 = BeanUtil.copyToList(orderBaseVOS,OrderBaseVO.class);
        }else

        if (year.equals("2024")) {

            List<OrderBase2024Es> orderBaseVOS = new ArrayList<>();
            SearchScrollHits<OrderBase2024Es> scrollHits = elasticsearchRestTemplate.searchScrollStart(
                    60000, // Scroll 上下文保持时间（毫秒）
                    query,
                    OrderBase2024Es.class,
                    IndexCoordinates.of("pg_order_base_2024")
            );

            String scrollId = scrollHits.getScrollId();
            int batch = 0;

            if (scrollHits.getTotalHits() > 60000)
                throw  new RemoteServiceException("数据超过六万上限，请分批导出");


            try {
                while (!scrollHits.isEmpty()) {
                    // 3. 处理当前批次数据
                    scrollHits.forEach(hit -> {
                        OrderBase2024Es entity = hit.getContent();
                        orderBaseVOS.add(entity);
                    });

                    batch++;

                    // 4. 获取下一批次
                    scrollHits = elasticsearchRestTemplate.searchScrollContinue(
                            scrollId,
                            60000,
                            OrderBase2024Es.class,
                            IndexCoordinates.of("pg_order_base_2024")
                    );
                    scrollId = scrollHits.getScrollId(); // 更新 scrollId
                }
            } finally {
                // 5. 清理 Scroll 上下文
                if (scrollId != null) {
                    List<String> scroll = new ArrayList<>();
                    scroll.add(scrollId);
                    elasticsearchRestTemplate.searchScrollClear(scroll);
                }
            }

            orderBaseVOS1 = BeanUtil.copyToList(orderBaseVOS,OrderBaseVO.class);
        }else


        if (year.equals("2022") || Convert.toInt(year) < 2022) {

            List<OrderBase2022Es> orderBaseVOS = new ArrayList<>();
            SearchScrollHits<OrderBase2022Es> scrollHits = elasticsearchRestTemplate.searchScrollStart(
                    60000, // Scroll 上下文保持时间（毫秒）
                    query,
                    OrderBase2022Es.class,
                    IndexCoordinates.of("pg_order_base_2022")
            );

            String scrollId = scrollHits.getScrollId();
            int batch = 0;

            if (scrollHits.getTotalHits() > 60000)
                throw  new RemoteServiceException("数据超过六万上限，请分批导出");


            try {
                while (!scrollHits.isEmpty()) {
                    // 3. 处理当前批次数据
                    scrollHits.forEach(hit -> {
                        OrderBase2022Es entity = hit.getContent();
                        orderBaseVOS.add(entity);
                    });

                    batch++;

                    // 4. 获取下一批次
                    scrollHits = elasticsearchRestTemplate.searchScrollContinue(
                            scrollId,
                            60000,
                            OrderBase2022Es.class,
                            IndexCoordinates.of("pg_order_base_2022")
                    );
                    scrollId = scrollHits.getScrollId(); // 更新 scrollId
                }
            } finally {
                // 5. 清理 Scroll 上下文
                if (scrollId != null) {
                    List<String> scroll = new ArrayList<>();
                    scroll.add(scrollId);
                    elasticsearchRestTemplate.searchScrollClear(scroll);
                }
            }

            orderBaseVOS1 = BeanUtil.copyToList(orderBaseVOS,OrderBaseVO.class);
        }else {

            List<OrderBaseEs> orderBaseVOS = new ArrayList<>();
            SearchScrollHits<OrderBaseEs> scrollHits = elasticsearchRestTemplate.searchScrollStart(
                    60000, // Scroll 上下文保持时间（毫秒）
                    query,
                    OrderBaseEs.class,
                    IndexCoordinates.of("pg_order_base")
            );

            String scrollId = scrollHits.getScrollId();
            int batch = 0;

            if (scrollHits.getTotalHits() > 60000)
                throw  new RemoteServiceException("数据超过六万上限，请分批导出");


            try {
                while (!scrollHits.isEmpty()) {
                    // 3. 处理当前批次数据
                    scrollHits.forEach(hit -> {
                        OrderBaseEs entity = hit.getContent();
                        orderBaseVOS.add(entity);
                    });

                    batch++;

                    // 4. 获取下一批次
                    scrollHits = elasticsearchRestTemplate.searchScrollContinue(
                            scrollId,
                            60000,
                            OrderBaseEs.class,
                            IndexCoordinates.of("pg_order_base")
                    );
                    scrollId = scrollHits.getScrollId(); // 更新 scrollId
                }
            } finally {
                // 5. 清理 Scroll 上下文
                if (scrollId != null) {
                    List<String> scroll = new ArrayList<>();
                    scroll.add(scrollId);
                    elasticsearchRestTemplate.searchScrollClear(scroll);
                }
            }

            orderBaseVOS1 = BeanUtil.copyToList(orderBaseVOS,OrderBaseVO.class);
        }

        List<String> orderBaseIds = orderBaseVOS1
                .stream()
                .map(OrderBaseVO::getId)
                .collect(Collectors.toList());

        List<WorkerOrder> workerOrderList = workerOrderService.lambdaQuery().in(WorkerOrder::getWorkerOrderId, orderBaseIds)
                .eq(WorkerOrder::getPayStatus, MaterialExamineStatusEnum.PAID.getKey())
                .select(WorkerOrder::getWorkerOrderId, WorkerOrder::getTotalAmount).list();

        for (OrderBaseVO orderBaseVO : orderBaseVOS1) {

            List<WorkerOrder> workerOrders = workerOrderList.stream()
                    .filter(item -> item.getWorkerOrderId().equals(orderBaseVO.getId())).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(workerOrderList)) {
                BigDecimal bigDecimal = workerOrders.stream().map(WorkerOrder::getTotalAmount).reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                orderBaseVO.setWorkerPrice(bigDecimal);
            }

        }

        return orderBaseVOS1;

    }
}
